
#!/bin/bash

set -e

export HOST_IP
export DOLTLAB_ENTERPRISE_ONLINE_PRODUCT_CODE
export DOLTLAB_ENTERPRISE_ONLINE_SHARED_KEY
export DOLTLAB_ENTERPRISE_ONLINE_API_KEY
export DOLTLAB_ENTERPRISE_ONLINE_LICENSE_KEY
export DOLTLAB_ENTERPRISE_HARDWARE_ID

USE_TLS=""

check_args() {
  if [[ "$#" -ge 1 ]]; then
    if [[ "$1" != "https" ]]; then
        echo "unrecognized argument: $1"
        exit 1
    else
      USE_TLS="true"
      echo "Preparing to serve DoltLab over HTTPS"
    fi
  fi
}

check_env() {
  if [[ -z "$HOST_IP" || -z "$DOLT_PASSWORD" || -z "$DOLTHUBAPI_PASSWORD" ]]; then
    echo "Must supply HOST_IP, DOLT_PASSWORD, and DOLTHUBAPI_PASSWORD"
    exit 1
  fi
  if [[ -z "$EMAIL_HOST" || -z "$EMAIL_PORT" || -z "$NO_REPLY_EMAIL" ]]; then
    echo "Must supply EMAIL_HOST, EMAIL_PORT, and NO_REPLY_EMAIL"
    exit 1
  fi
  if [[ "$USE_TLS" == "true" ]]; then
      if [[ -z "$TLS_CERT_CHAIN" || -z "$TLS_PRIVATE_KEY" ]]; then
        echo "Must supply TLS_CERT_CHAIN and TLS_PRIVATE_KEY"
        exit 1
      fi
  fi

  check_email_auth_method
}

check_email_auth_method() {
  if [ -z "$EMAIL_AUTH_METHOD" ]; then
    export EMAIL_AUTH_METHOD=plain
  fi

  if [ "$EMAIL_AUTH_METHOD" == "plain" ] || [ "$EMAIL_AUTH_METHOD" == "login" ]
  then
    check_plain_login_auth_env
  elif [ "$EMAIL_AUTH_METHOD" == "oauthbearer" ]
  then
    check_oauthbearer_auth_env
  fi
}

check_plain_login_auth_env() {
  if [[ -z "$EMAIL_USERNAME" || -z "$EMAIL_PASSWORD" ]]; then
    echo "Must supply EMAIL_USERNAME and EMAIL_PASSWORD"
    exit 1
  fi
}

check_oauthbearer_auth_env() {
  if [[ -z "$EMAIL_USERNAME" || -z "$EMAIL_OAUTH_TOKEN" ]]; then
    echo "Must supply EMAIL_USERNAME and EMAIL_OAUTH_TOKEN"
    exit 1
  fi
}

create_token_keys() {
    chmod +x ./gentokenenckey
    create_token_keys_for_service "doltlabremoteapi"
    create_token_keys_for_service "doltlabapi"
    create_token_keys_for_service "doltlabfileserviceapi"
}

create_token_keys_for_service() {
  ./gentokenenckey > "$1"_iter_token.keys
}

create_envoy_config() {
  local tmpl="$(pwd)"/envoy.tmpl

  if [[ "$USE_TLS" == "true" ]]; then
    tmpl="$(pwd)"/envoy-tls.tmpl
  fi

  cat "$tmpl" | envsubst \$HOST_IP > envoy.yaml
}

update_images() {
  local config="$(pwd)"/docker-compose.yaml

  if [[ "$USE_TLS" == "true" ]]; then
    config="$(pwd)"/docker-compose-tls.yaml
  fi

  docker-compose -f "$config" pull
}

start_services() {
  local config="$(pwd)"/docker-compose.yaml

  if [[ "$USE_TLS" == "true" ]]; then
    config="$(pwd)"/docker-compose-tls.yaml
  fi

  DOLT_PASSWORD="$DOLT_PASSWORD" \
  DOLTHUBAPI_PASSWORD="$DOLTHUBAPI_PASSWORD" \
  docker-compose -f "$config" up -d
}

enable_usage_metrics() {
    if [ -z "$DOLTHUB_METRICS_ENABLED" ]; then
      echo "Enabling DoltLab usage metrics."
      export DOLTHUB_METRICS_ENABLED=true
    fi
}

send_deployment_metrics() {
  if [ -n "$DOLTHUB_METRICS_ENABLED" ]; then
    chmod +x send_doltlab_deployed_event
    ./send_doltlab_deployed_event --version v2.0.0
  fi
}

run_with_metrics() {
  enable_usage_metrics
  send_deployment_metrics
}

set_env() {
  set_default_user_env
  set_email_smtp_env
}

set_default_user_env() {
  if [ -z "$DEFAULT_USER" ]; then
    echo "Setting DEFAULT_USER=admin"
    export DEFAULT_USER=admin
  fi
  if [ -z "$DEFAULT_USER_PASSWORD" ]; then
    echo "Setting DEFAULT_USER_PASSWORD=DoltLab1234"
    export DEFAULT_USER_PASSWORD=DoltLab1234
  fi
  if [ -z "$DEFAULT_USER_EMAIL" ]; then
    echo "Setting DEFAULT_EMAIL=$NO_REPLY_EMAIL"
    export DEFAULT_USER_EMAIL="$NO_REPLY_EMAIL"
  fi
}

set_email_smtp_env() {
  if [ -z "$EMAIL_CLIENT_HOSTNAME" ]; then
    echo "Setting EMAIL_CLIENT_HOSTNAME=localhost"
    export EMAIL_CLIENT_HOSTNAME=localhost
  fi
}

create_doltlab_network_if_not_exists() {
  docker network inspect doltlab >/dev/null 2>&1 || \
      docker network create --driver bridge doltlab
}

load_config() {
  export ADMIN_CONFIG=${ADMIN_CONFIG-./admin-config.yaml}
  if [ -f "$ADMIN_CONFIG" ]; then
    echo "Loading administrator config $ADMIN_CONFIG"
    ./config_loader -config "$ADMIN_CONFIG"
    source ./doltlab-config.vars
  fi
}

_main() {
    check_args "$@"
    check_env
    set_env
    create_doltlab_network_if_not_exists
    create_token_keys
    create_envoy_config
    run_with_metrics
    load_config
    update_images
    start_services
    echo "DoltLab Documentation is available at https://docs.dolthub.com/guides/doltlab"
}

_main "$@"

